@@ -2,7 +2,7 @@
 
 cart-review checkout
 
-$Id: checkout.py 3302 2010-01-21 21:34:49Z larubbio $
+$Id: checkout.py 3695 2010-06-21 06:26:39Z taito.horiuchi $
 """
 
 
@@ -14,15 +14,11 @@ from datetime import timedelta
 from zope.event import notify
 from zope.formlib import form
 from zope import schema, interface
-from zope.app.event.objectevent import ObjectCreatedEvent
-from zope.app.renderer.plaintext import PlainTextToHTMLRenderer
-from zope.app.component.hooks import getSite
+
+from zope.lifecycleevent import ObjectCreatedEvent
 
 from zope import component
 
-from zope.schema.interfaces import IField
-from zope.app.apidoc import interface as apidocInterface
-
 from zc.table import column
 from getpaid.wizard import Wizard, ListViewController, interfaces as wizard_interfaces
 from getpaid.core import interfaces, options, cart
@@ -33,21 +29,24 @@ from AccessControl import getSecurityManager
 from ZTUtils import make_query
 from ZTUtils.Zope import complex_marshal
 
-from Products.Five.formlib import formbase
 from Products.Five.browser import BrowserView
+from Products.Five.browser.decode import processInputs
 from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile
 from Products.CMFCore.utils import getToolByName
+from Products.CMFPlone.i18nl10n import utranslate
 
-# Bruno - missing import for INamedOrderUtility
 from Products.PloneGetPaid.interfaces import INamedOrderUtility
 
 from Products.PloneGetPaid.interfaces import IGetPaidManagementOptions, IAddressBookUtility
 from Products.PloneGetPaid.i18n import _
-from Products.PloneGetPaid import config
+from Products.PloneGetPaid.browser.cart import formatLinkCell
 
 from base import BaseFormView
 import cart as cart_core
 from widgets import CountrySelectionWidget, StateSelectionWidget, CCExpirationDateWidget
+from Products.PloneGetPaid.browser.interfaces import IDontShowGetPaidPortlets
+
+from Products.CMFPlone.utils import safe_unicode
 
 def null_condition( *args ):
     return ()
@@ -74,7 +73,7 @@ def make_hidden_input(*args, **kwargs):
     # again.  Somewere that populated array is replaced with an empty array
     # setting the shipping cost to 0.  This only happens if the site has
     # shipping set up and the user hits 'Back' from the review and pay step
-# This 'fix' actually results in shipping always being 0. 
+# This 'fix' actually results in shipping always being 0.
 #    if kwargs.has_key('form.shipping_method_code'):
 #        del kwargs['form.shipping_method_code']
 
@@ -93,7 +92,7 @@ def make_hidden_input(*args, **kwargs):
         except UnicodeDecodeError:
             v = str(v)
         qlist[i] = ('<input type="hidden" name="%s%s" value="%s">'
-                    % (hq(k), m, hq(v)))
+                    % (hq(k), m, safe_unicode(hq(v))))
 
     return '\n'.join(qlist)
 
@@ -109,7 +108,7 @@ class BaseCheckoutForm( BaseFormView ):
     _next_url = None # redirect url
     wizard = None  # wizard
 
-    interface.implements( wizard_interfaces.IWizardFormStep )
+    interface.implements( wizard_interfaces.IWizardFormStep, IDontShowGetPaidPortlets )
 
     def __init__( self, context, request ):
         self.context = context
@@ -264,6 +263,7 @@ class CheckoutWizard( Wizard ):
     update / render methods directly.
     """
 
+
     def checkShoppingCart(self):
         cart = component.getUtility(interfaces.IShoppingCartUtility).get( self.context )
         if cart is None or not len(cart):
@@ -309,7 +309,6 @@ class CheckoutWizard( Wizard ):
                                                order_id = order_id,
                                                creation_date = timedelta(1) )
                 if len(results) == 1:
-                    order = list( results )[0]
                     base_url = self.context.absolute_url()
                     url = base_url + '/@@getpaid-thank-you?order_id=%s' %(order_id)
 
@@ -368,13 +367,6 @@ class CheckoutController( ListViewController ):
     conditions = {'checkout-select-shipping' : 'checkShippableCart'}
     steps = ['checkout-address-info', 'checkout-select-shipping', 'checkout-review-pay']
 
-    def getStep( self, step_name ):
-        step = component.getMultiAdapter(
-                    ( self.wizard.context, self.wizard.request ),
-                    name=step_name
-                    )
-        return step.__of__( Acquisition.aq_inner( self.wizard.context ) )
-
     def checkShippableCart( self ):
         cart_utility = component.getUtility( interfaces.IShoppingCartUtility )
         cart = cart_utility.get( self.wizard.context )
@@ -411,8 +403,9 @@ class CheckoutController( ListViewController ):
 
 class CheckoutAddress( BaseCheckoutForm ):
     """
-    browser view for collecting credit card information and submitting it to
+    browser view for collecting addresses information and submitting it to
     a processor.
+
     """
 
     sections = ('billing_address','shipping_address','contact_information')
@@ -426,7 +419,7 @@ class CheckoutAddress( BaseCheckoutForm ):
     template = ZopeTwoPageTemplateFile("templates/checkout-address.pt")
 
     def update( self ):
-        formbase.processInputs( self.request )
+        processInputs( self.request )
         self.adapters = self.wizard.data_manager.adapters
         super( CheckoutAddress, self).update()
 
@@ -545,7 +538,7 @@ class CheckoutReviewAndPay( BaseCheckoutForm ):
 
     columns = [
         column.GetterColumn( title=_(u"Quantity"), getter=cart_core.LineItemColumn("quantity") ),
-        column.GetterColumn( title=_(u"Name"), getter=cart_core.lineItemURL ),
+        column.GetterColumn( title=_(u"Name"), getter=cart_core.lineItemURL, cell_formatter=formatLinkCell ),
         column.GetterColumn( title=_(u"Price"), getter=cart_core.lineItemPrice ),
         column.GetterColumn( title=_(u"Total"), getter=cart_core.lineItemTotal ),
        ]
@@ -593,7 +586,9 @@ class CheckoutReviewAndPay( BaseCheckoutForm ):
 
         for column in self.columns:
             if hasattr(column, 'title'):
-                column.title = getSite().translate(msgid=column.title, domain='plonegetpaid')
+                column.title = utranslate(domain='plonegetpaid',
+                                          msgid=column.title,
+                                          context=self.request)
 
         # create an order so that tax/shipping utilities have full order information
         # to determine costs (ie. billing/shipping address ).
@@ -609,7 +604,7 @@ class CheckoutReviewAndPay( BaseCheckoutForm ):
         formatter.cssClasses['table'] = 'listing'
         return formatter()
 
-    @form.action(_(u"Make Payment"), name="make-payment", condition=form.haveInputWidgets )
+    @form.action(_(u"Make Payment"), name="make-payment")#, condition=form.haveInputWidgets )
     def makePayment( self, action, data ):
         """ create an order, and submit to the processor
         """
@@ -632,7 +627,10 @@ class CheckoutReviewAndPay( BaseCheckoutForm ):
         # extract data to our adapters
 
         formSchemas = component.getUtility(interfaces.IFormSchemas)
-        order.user_payment_info_last4 = adapters[formSchemas.getInterface('payment')].credit_card[-4:]
+        last4 = None
+        if adapters[formSchemas.getInterface('payment')].credit_card:
+            last4 = adapters[formSchemas.getInterface('payment')].credit_card[-4:]
+        order.user_payment_info_last4 = last4
         order.name_on_card = adapters[formSchemas.getInterface('payment')].name_on_card
         order.bill_phone_number = adapters[formSchemas.getInterface('payment')].bill_phone_number
         result = processor.authorize( order, adapters[formSchemas.getInterface('payment')] )
@@ -676,8 +674,6 @@ class CheckoutReviewAndPay( BaseCheckoutForm ):
 
 
     def createOrder( self ):
-        order_manager = component.getUtility( interfaces.IOrderManager )
-
         order = self.createTransientOrder()
 
         shipping_code = self.wizard.data_manager.get('form.shipping_method_code')
@@ -694,8 +690,13 @@ class CheckoutReviewAndPay( BaseCheckoutForm ):
             order.shipping_cost = shipping_method_obj.cost
             order.shipments = {} # XXX not implemented
 
+
         notify( ObjectCreatedEvent( order ) )
 
+        if order.shopping_cart.is_recurring():
+            first_item = order.shopping_cart.values()[0]
+            order.renewal_date = order.creation_date + timedelta(30 * float(first_item.interval))
+
         return order
 
     def getNextURL(self, order):
@@ -715,12 +716,7 @@ class CheckoutReviewAndPay( BaseCheckoutForm ):
                      f_states.REVIEWING,
                      f_states.CHARGED):
             return base_url + '/@@getpaid-thank-you?order_id=%s&finance_state=%s' %(order.order_id, state)
-            
-    def isPlone3(self):
-        """test if it is a plone3 site
-        """
-        return config.PLONE3
-            
+
 
 class ShippingRate( options.PropertyBag ):
     title = "Shipping Rate"
@@ -793,6 +789,9 @@ class OrderFormatter( cart_core.CartFormatter ):
     def getTotals( self ):
         return OrderTotals( self.context, self.request)
 
+    def is_recurring(self):
+        return self.context.shopping_cart.is_recurring()
+
 class OrderTotals( cart.CartItemTotals ):
 
     interface.implements( interfaces.ILineContainerTotals )
@@ -835,8 +834,8 @@ class StorePropertyView(BrowserView):
         settings = IGetPaidManagementOptions(portal)
         value = getattr( settings, name, '')
         if value:
-            renderer = PlainTextToHTMLRenderer(value, self.request)
-            value = renderer.render().strip()
+            transforms = getToolByName(self.context, 'portal_transforms')
+            value = transforms('web_intelligent_plain_text_to_html', value).strip()
         return value
 
 class DisclaimerView( StorePropertyView ):
@@ -905,7 +904,7 @@ class AddressBookView(BrowserView):
         for entry in self.getEntryNames().keys():
 
             field_assignations += "if (contact_name == '%s') {\n" % entry
-            for name in apidocInterface.getElements(interfaces.IShippingAddress, type=IField).keys():
+            for name in schema.getFieldsNames(interfaces.IShippingAddress):
                 try:
                     if name == "ship_state":
                         field_assignations += """\n setTimeout("doWaitUntilStatesAreLoaded('%s')",2000);""" %  getattr(addressBookUsr[entry],name) or ''
